/* Yet Another Forum.NET
 * Copyright (C) 2003-2005 Bjørnar Henden
 * Copyright (C) 2006-2013 Jaben Cargman
 * Copyright (C) 2014-2015 Ingo Herbote
 * http://www.yetanotherforum.net/
 * 
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at

 * http://www.apache.org/licenses/LICENSE-2.0

 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

namespace YAF.Core
{
  #region Using

  using System;

  using YAF.Types;
  using YAF.Types.Extensions;
  using YAF.Utils;

  #endregion

  /// <summary>
  /// The moderate forum page.
  /// </summary>
  public class ModerateForumPage : ForumPage
  {
    #region Constructors and Destructors

      /// <summary>
      /// Initializes a new instance of the <see cref="ModerateForumPage"/> class.
      /// </summary>
      public ModerateForumPage()
      : this(null)
    {
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="ModerateForumPage"/> class.
    /// </summary>
    /// <param name="transPage">
    /// The trans page.
    /// </param>
    public ModerateForumPage([CanBeNull] string transPage)
      : base(transPage)
    {
    }

    #endregion

    #region Properties

    /// <summary>
    /// Gets PageName.
    /// </summary>
    public override string PageName
    {
      get
      {
        return "moderate_{0}".FormatWith(base.PageName);
      }
    }

    #endregion

    #region Methods

    /// <summary>
    /// The page_ load.
    /// </summary>
    /// <param name="sender">
    /// The sender.
    /// </param>
    /// <param name="e">
    /// The e.
    /// </param>
    protected void Page_Load([NotNull] object sender, [NotNull] EventArgs e)
    {
      // Only moderators are allowed here
      if (!this.PageContext.ForumModeratorAccess || !this.PageContext.IsModeratorInAnyForum)
      {
        YafBuildLink.AccessDenied();
      }
    }

    #endregion
  }
}